/*
    Copyright 2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include "blowfish.h"

typedef unsigned char   u8;



#define VER         "0.1"
#define MAINREG     "Software\\nanoCom Corporation\\iSpQ VideoChat "



u_char *base64_decode(u_char *data, int *size);
void ispq_decrypt(void);
void wait_exit(int ret);
void std_err(void);



int main(void) {
    fputs("\n"
        "ISPQ sha1 password decrypter "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stderr);

    printf(
        "note: ISPQ saves only the sha1 hash and NOT the original password.\n"
        "      anyway due to the type of login mechanism adopted by ISPQ this hash can\n"
        "      still be used to login because it's used at the place of the original\n"
        "      password (strLoginPassword) without salts.\n"
        "\n");

    ispq_decrypt();

    wait_exit(0);
    return(0);
}



int regkeyr(HKEY hKey, LPCTSTR lpSubKey, LPTSTR lpValueName, char *buff, DWORD len) {
    HKEY    key;

    buff[0] = 0;
    if(RegOpenKeyEx(hKey, lpSubKey, 0, KEY_READ, &key) != ERROR_SUCCESS) {
        return(-1);
    }
    if(RegQueryValueEx(key, lpValueName, NULL, REG_NONE, buff, &len) != ERROR_SUCCESS) {
        RegCloseKey(key);
        return(-1);
    }
    RegCloseKey(key);
    buff[len] = 0;
    return(len);
}



u_char *base64_decode(u_char *data, int *size) {
    int     len,
            xlen,
            a,
            b,
            c;
    u_char  *buff,
            *limit,
            *p;
    static const u_char base[128] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x3e, 0x00, 0x00, 0x00, 0x3f,
        0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b,
        0x3c, 0x3d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
        0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
        0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
        0x17, 0x18, 0x19, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20,
        0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
        0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30,
        0x31, 0x32, 0x33, 0x00, 0x00, 0x00, 0x00, 0x00
    };

    if(!size || (*size < 0)) {      // use size -1 for auto size!
        len = strlen(data);
    } else {
        len = *size;
    }
    xlen = ((len >> 2) * 3) + 1;    // NULL included in output for text
    buff = malloc(xlen);
    if(!buff) return(NULL);

    p = buff;
    limit = data + len;
    for(;;) {
#define B64NEXT do {            /* auto chars cleaning */   \
                    c = *data;                              \
                    if((data == limit) || (c == '=')) {     \
                        c = 0;                              \
                        break;                              \
                    }                                       \
                    data++;                                 \
                } while(c && ((c <= ' ') || (c > 0x7f)));   \
                if(!c) break;

        B64NEXT;
        a = base[c];

        B64NEXT;
        b = base[c];
        *p++ = (a << 2)        | (b >> 4);

        B64NEXT;
        a = base[c];
        *p++ = ((b & 15) << 4) | (a >> 2);

        B64NEXT;
        *p++ = ((a & 3) << 6)  | base[c];

#undef B64NEXT
    }
    *p = 0;

    len = p - buff;
    if(size) *size = len;
    if((len + 1) != xlen) {             // save memory
        buff = realloc(buff, len + 1);  // NULL included!
        if(!buff) return(NULL);
    }
    return(buff);
}



void ispq_decrypt(void) {
    blf_ctx ctx;
    int     i,
            userlen,
            pwdlen,
            bufflen,
            account,
            vernum;
    u8      regpath[1024],
            buff[1024],
            tmp[64],
            *user,
            *pwd;

    for(vernum = 1; vernum < 10; vernum++) {    // more than enough since the current version is 8
        for(account = 1; ; account++) {
            sprintf(regpath, "%s%d\\%s", MAINREG, vernum, "Lists\\RecentLogin");
            sprintf(tmp, "%d", account);
            if(regkeyr(HKEY_CURRENT_USER, regpath, tmp, buff, sizeof(buff)) < 0) break;

            bufflen = strlen(buff);
            pwdlen = 64;
            if(bufflen < pwdlen) continue;
            pwd = buff + bufflen - pwdlen;

            for(user = buff; *user && (*user == '#'); user++);
            userlen = pwd - user;

            for(i = 0; i < pwdlen; i++) {
                if(pwd[i] == '*') pwd[i] = '+';
            }
            pwd = base64_decode(pwd, &pwdlen);

            user[userlen] = 0;
            printf("  username             %s\n", user);

            sprintf(regpath, "%s%d\\Accounts\\%s\\General", MAINREG, vernum, user);
            if(regkeyr(HKEY_CURRENT_USER, regpath, "ID", user + userlen, sizeof(buff) - ((user + userlen) - buff)) < 0) {
                printf("  password can't be decrypted, no ID found\n");
                continue;
            }

            blf_key(&ctx, user, strlen(user));
            blf_dec(&ctx, (void *)pwd, pwdlen >> 3);
            pwd[40] = 0;
            printf("  password sha1 hash   %s\n\n", pwd);
            free(pwd);
        }
    }
}



void wait_exit(int ret) {
    fprintf(stderr, "\n  press RETURN to exit\n");
    fgetc(stdin);
    exit(ret);
}



void std_err(void) {
    perror("\nError");
    wait_exit(1);
}


